// ds3231.c
// 2018.04.10
// Programm fuer MIDI-Block 
// Version fuer die Pruefung von Uhr DS3231
#include <avr/io.h>	// Muss immer sein
#include <avr/pgmspace.h> 	// Fuer Flash

#include "main.h" 	// Allgemeine Programmablauf
#include "i2c_prog.h"	// I2C
#include "ds3231.h"	// Uhr
#include "lcd.h"	// LCD

unsigned char ds3231_hour;
unsigned char ds3231_min;
unsigned char ds3231_sec;

unsigned char ds3231_day;
unsigned char ds3231_wday;
unsigned char ds3231_month;
unsigned char ds3231_year;

unsigned int  ds3231_temp;
unsigned char ds3231_t1;
unsigned char ds3231_t2;

unsigned char ds3231_alarm_hour;
unsigned char ds3231_alarm_min;
unsigned char ds3231_alarm_on;

//      
const uint8_t ds3231_alarm[] PROGMEM   = {0x02,0x04,0x04,0x0E,0x0E,0x1F,0x1F,0x04}; 

//    (   ) ====================================================
unsigned char ds3231_bcd (unsigned char data)	// bcd > binary
{
	unsigned char bc;

	bc=((((data&(1<<6))|(data&(1<<5))|(data&(1<<4)))*0x0A)>>4)+((data&(1<<3))|(data&(1<<2))|(data&(1<<1))|(data&0x01));

	return bc;
} 

unsigned char ds3231_bin(unsigned char dec){ // binary > bcd

	unsigned char bcd=0,dig,num,count=0;
	num = dec;
	for(int i=0;i<4;i++){
		dig = num%10;
		num = num/10;
		bcd = (dig<<count)|bcd;
		count += 4;
	}

	return bcd;
}

//    =======================================================================================================

//   
void ds3231_RTC_init(void){

	i2c_start_cond();               //  i2c
	i2c_send_byte(ds3231_RTC_adr_write);   //   ,  
	i2c_send_byte(0x0E);	        //    
	i2c_send_byte(0b00100000);      //       1 
	i2c_send_byte(0b00001000);      //   32 
	i2c_stop_cond();                //  i2c

}

//    
void ds3231_RTC_read_time(void){

	i2c_start_cond();               //  i2c
	i2c_send_byte(ds3231_RTC_adr_write);   //   ,  
	i2c_send_byte(0x00);	        //    
	i2c_stop_cond();                //  i2c

	i2c_start_cond();               //  i2c
	i2c_send_byte(ds3231_RTC_adr_read);    //   ,  
	ds3231_sec = ds3231_bcd(i2c_get_byte(0));     //  , ACK
	ds3231_min = ds3231_bcd(i2c_get_byte(0));     //  , ACK
	ds3231_hour = ds3231_bcd(i2c_get_byte(0));    //  , ACK
	ds3231_wday = ds3231_bcd(i2c_get_byte(0));    //   , ACK
	ds3231_day = ds3231_bcd(i2c_get_byte(0));     //  , ACK
	ds3231_month = ds3231_bcd(i2c_get_byte(0));   //  , ACK
	ds3231_year = ds3231_bcd(i2c_get_byte(1));    //  , NACK
	i2c_stop_cond();                //  i2c

}

//  
void ds3231_RTC_write_time(unsigned char hour1,unsigned char min1, unsigned char sec1){

    i2c_start_cond();               //  i2c
    i2c_send_byte(ds3231_RTC_adr_write);   //   ,  
    i2c_send_byte(0x00);	        //    
    i2c_send_byte(ds3231_bin(sec1));       // 0x00  (     ?)
	i2c_send_byte(ds3231_bin(min1));       // 0x01 
    i2c_send_byte(ds3231_bin(hour1));      // 0x02 
    i2c_stop_cond();                //  i2c

}

//  
void ds3231_RTC_write_date(unsigned char wday, unsigned char day, unsigned char month, unsigned char year){

	i2c_start_cond();               //  i2c
    i2c_send_byte(ds3231_RTC_adr_write);   //   ,  
    i2c_send_byte(0x03);	        //    
    i2c_send_byte(ds3231_bin(wday));       // 0x03   ( - 1,  2,  3,  4,  5,  6,  7)
    i2c_send_byte(ds3231_bin(day));        // 0x04  
	i2c_send_byte(ds3231_bin(month));      // 0x05 
	i2c_send_byte(ds3231_bin(year));       // 0x06 
    i2c_stop_cond();                //  i2c
}

//  
void ds3231_RTC_read_temper(void){


	i2c_start_cond();               //  i2c
	i2c_send_byte(ds3231_RTC_adr_write);   //   ,  
	i2c_send_byte(0x11);	        //    
	i2c_stop_cond();                //  i2c

	i2c_start_cond();               //  i2c
	i2c_send_byte(ds3231_RTC_adr_read);    //   ,  
	ds3231_t1 = i2c_get_byte(0);           //  MSB 
	ds3231_t2 = i2c_get_byte(1);           //  LSB 
	i2c_stop_cond();                //  i2c

//	ds3231_t2=(ds3231_t2>>7); 					//   6 -  0,25 (2 )
	                    			//   7 -  0,5 (1 )
	ds3231_t2=(ds3231_t2>>6);
//	ds3231_t2=ds3231_t2*5;

}

//   
void ds3231_put_wday (unsigned char platz) {
if (ds3231_wday==1) {lcd_puff_string_P(PSTR("Sonntag   "), platz);}
if (ds3231_wday==2) {lcd_puff_string_P(PSTR("Montag    "), platz);}
if (ds3231_wday==3) {lcd_puff_string_P(PSTR("Dienstag  "), platz);}
if (ds3231_wday==4) {lcd_puff_string_P(PSTR("Mittwoch  "), platz);}
if (ds3231_wday==5) {lcd_puff_string_P(PSTR("Donnerstag"), platz);}
if (ds3231_wday==6) {lcd_puff_string_P(PSTR("Freitag   "), platz);}
if (ds3231_wday==7) {lcd_puff_string_P(PSTR("Samstag   "), platz);}
}

void ds3231_put_time (unsigned char platz) {
	ds3231_RTC_read_time();

	lcd_puff_data(0x30+(ds3231_hour / 10),platz);
	lcd_puff_data(0x30+(ds3231_hour % 10),(platz+1));
	lcd_puff_data(':',(platz+2));
	lcd_puff_data(0x30+(ds3231_min / 10),(platz+3));
	lcd_puff_data(0x30+(ds3231_min % 10),(platz+4));
	lcd_puff_data(':',(platz+5));
	lcd_puff_data(0x30+(ds3231_sec / 10),(platz+6));
	lcd_puff_data(0x30+(ds3231_sec % 10),(platz+7));

}

void ds3231_put_temperatur (unsigned char platz) {
	unsigned char a,b;
	
	ds3231_RTC_read_temper();

	lcd_puff_data('+',platz);

	
	a = ds3231_t1;
	b = 0;
	while(a >= 10){
		b++;
        a -= 10; 
	}
	if (b) b += 48;
    else b = ' ';
	lcd_puff_data(b,(platz+1));
	lcd_puff_data((a+48),(platz+2));

	lcd_puff_data(',',(platz+3));

	switch(ds3231_t2){
		case 0 : lcd_puff_data('2',(platz+4)); lcd_puff_data('5',(platz+5)); break;
		case 1 : lcd_puff_data('5',(platz+4)); lcd_puff_data(' ',(platz+5)); break;
		case 2 :  lcd_puff_data('7',(platz+4)); lcd_puff_data('5',(platz+5)); break;
		default :  lcd_puff_data('0',(platz+4)); lcd_puff_data(' ',(platz+5));
	}	
}

//    
void ds3231_temp_start(void){
	i2c_start_cond();               //  i2c
	i2c_send_byte(ds3231_RTC_adr_write);   //   ,  
	i2c_send_byte(0x0E);	        //    
	i2c_send_byte(0b00100000);      //       1 
	i2c_stop_cond();                //  i2c
}

//  
void ds3231_hour_plus(void){
	ds3231_hour++;
	if(ds3231_hour > 23) ds3231_hour = 0;

	ds3231_RTC_write_time(ds3231_hour,ds3231_min,ds3231_sec);

	ds3231_RTC_read_time();
	ds3231_put_time(61);
}

//  
void ds3231_min_plus(void){
	ds3231_min++;
	if(ds3231_min > 59) ds3231_min = 0;

	ds3231_RTC_write_time(ds3231_hour,ds3231_min,ds3231_sec);

	ds3231_RTC_read_time();
	ds3231_put_time(61);
}

//  
void ds3231_sec_res(void){
	ds3231_RTC_read_time();	
	if(ds3231_sec > 30){
		ds3231_min++;
		if(ds3231_min > 59){
			ds3231_min = 0;
			ds3231_hour++;
		}
	}
	ds3231_sec = 0;

	ds3231_RTC_write_time(ds3231_hour,ds3231_min,ds3231_sec);

	ds3231_RTC_read_time();
	ds3231_put_time(61);
}

//   
void ds3231_wday_plus(void){
	ds3231_wday++;
	if(ds3231_wday > 7) ds3231_wday = 1;

	ds3231_RTC_write_date(ds3231_wday,ds3231_day,ds3231_month,ds3231_year);

	ds3231_RTC_read_time();
	ds3231_put_wday(41);
}
