// i2c_prog.c   Programm-i2c Master
// 2018.04.10
// Programm fuer MIDI-Block 
// Version fuer die Pruefung von Uhr DS3231
#include <avr/io.h>	// Muss immer sein
#include <util/delay.h>		// Fuer Verzoegerungen

#include "main.h" 	// Allgemeine Programmablauf
#include "i2c_prog.h"	// I2C

volatile unsigned char i2c_frame_error=0; /// wenn > 0, war frame-Fehler 
// (nach STOP sind SDA und SCL nicht HIGH geworden. 

void i2c_stop_cond (void)	{
	I2C_DDR |= (1<<I2C_SCL); //  SCL (.0)
	I2C_DELAY_LOW
	I2C_DDR |= (1<<I2C_SDA); //  SDA (.0)
	I2C_DELAY_HIGH
		
	I2C_DDR &= ~(1<<I2C_SCL); //  SCL (.1)
	I2C_DELAY_MID
	I2C_DDR &= ~(1<<I2C_SDA); //  SDA (.1)
	I2C_DELAY_HIGH
	
	//  -
	i2c_frame_error=0;									// reset fuer framefehler-Zaehler
	if ((I2C_PIN & (1<<I2C_SDA)) == 0) i2c_frame_error++;	// Wenn SDA !=1, Fehler
	if ((I2C_PIN & (1<<I2C_SCL)) == 0) i2c_frame_error++;	// Wenn SCL !=1, Fehler
	I2C_DELAY_HIGH
	I2C_DELAY_HIGH
	I2C_DELAY_HIGH
	I2C_DELAY_HIGH
}

void i2c_init (void) {
	I2C_DDR &= ~(1<<I2C_SDA);		// wenn DDR=0, haengt SDA an Widerstand, wenn DDR=1, so wird SDA an die Erde gezogen
	I2C_DDR &= ~(1<<I2C_SCL);		// wenn DDR=0, haengt SCL an Widerstand, wenn DDR=1, so wird SCL an die Erde gezogen
	I2C_PORT &= ~(1<<I2C_SDA);      // PORT=0 bedeutet, dass wenn DDR=1, wird SDA an die Erde gezogen, wenn DDR=0, so haengt SDA an Widerstand
	I2C_PORT &= ~(1<<I2C_SCL);      // PORT=0 bedeutet, dass wenn DDR=1, wird SCL an die Erde gezogen, wenn DDR=0, so haengt SCL an Widerstand
	i2c_stop_cond();   // Stop fuer Bus zum Verdacht
	i2c_stop_cond();   // Stop fuer Bus zum Verdacht
}


void i2c_start_cond (void)	{	
	I2C_DDR |= (1<<I2C_SDA); // SDA ziehen (logische.0)
	I2C_DELAY_MID
	I2C_DDR |= (1<<I2C_SCL); // SCL ziehen (logische 0)
	I2C_DELAY_HIGH
}

void i2c_restart_cond (void)	{	
	I2C_DDR &= ~(1<<I2C_SDA); // SDA lassen (logische 1)
	I2C_DELAY_HIGH
	I2C_DDR &= ~(1<<I2C_SCL); // SCL lassen (logische 1)
	I2C_DELAY_HIGH
	
	I2C_DDR |= (1<<I2C_SDA); // SDA ziehen (logische.0)
	I2C_DELAY_MID
	I2C_DDR |= (1<<I2C_SCL); // SCL ziehen (logische 0)
	I2C_DELAY_HIGH
}

unsigned char i2c_send_byte (unsigned char data)	{	
 unsigned char i;
 unsigned char ack=1;			// ACK, wenn ACK=1  Fehler
	for (i=0;i<8;i++)
	{
		if ((data & 0x80) == 0x00) I2C_DDR |= (1<<I2C_SDA);	// bit -> SDA (log.0)
		else I2C_DDR &= ~(1<<I2C_SDA); // log.1
		I2C_DELAY_HIGH
		I2C_DDR &= ~(1<<I2C_SCL);	// Impuls auf SCL
		I2C_DELAY_LOW
		I2C_DDR |= (1<<I2C_SCL); // SCL = 0
		data=data<<1;
	}
	I2C_DDR &= ~(1<<I2C_SDA); // SDA lassen (logische 1), um ACK empfangen zu koennen
	I2C_DELAY_HIGH
	I2C_DDR &= ~(1<<I2C_SCL); // SCL lassen (logische 1)
	I2C_DELAY_HIGH
	if ((I2C_PIN & (1<<I2C_SDA)) == (1<<I2C_SDA)) ack=1; else ack=0;	// ACK lesen

	I2C_DDR |= (1<<I2C_SCL); // SCL = 0
	
	return ack; // ACK (0) NACK (1) zurueckgeben
}

unsigned char i2c_get_byte (unsigned char last_byte)	{	
 unsigned char i, res=0;
	I2C_DDR &= ~(1<<I2C_SDA); // SDA lassen (logische 1)

	for (i=0;i<8;i++)
	{
		res=res<<1;
		I2C_DDR &= ~(1<<I2C_SCL); // SCL lassen (logische 1)		// Impuls auf SCL
		I2C_DELAY_HIGH
		if ((I2C_PIN&(1<<I2C_SDA))==(1<<I2C_SDA)) res=res|0x01; // SDA in res lesen
		I2C_DDR |= (1<<I2C_SCL); // SCL ziehen (logische 0)
		I2C_DELAY_MID
	}

	if (last_byte==0) I2C_DDR |= (1<<I2C_SDA); // SDA ziehen (logische 0), Besaetigung mit ACK, werden weiter lesen
	else I2C_DDR &= ~(1<<I2C_SDA); // SDA lassen (logische 1) ohne Bestaetigung, NACK, das war letzte byte
	I2C_DELAY_HIGH
	I2C_DDR &= ~(1<<I2C_SCL); // SCL lassen (logische 1)
	I2C_DELAY_LOW
	I2C_DDR |= (1<<I2C_SCL); // SCL ziehen (logische 0)
	I2C_DELAY_MID
	I2C_DDR &= ~(1<<I2C_SDA); // SDA lassen (logische 1)

	return res; // gelesene Wert zurueckgeben
}

