// meine_verzoegerung.h

#ifndef MEINE_VERZOEGERUNG_H
#define MEINE_VERZOEGERUNG_H 1

#ifndef F_CPU
#define F_CPU 16000000UL		// CPU_F fuer Verzoegerungen
#endif

/* Macro */

#define meine_verzoegerung_delms(x) (meine_verzoegerung_delayloop24((x*(F_CPU/5000))-1))  
/* x, ms, fuer delayloop24 */
		/* max. bei  F_CPU= 20 MHz 4194, d.h. 4,194 s. */
		/* max. bei  F_CPU= 16 MHz 5242, d.h. 5,242 s. */
		/* max. bei  F_CPU= 10 MHz 8388, d.h. 8,388 s. */
		/* max. bei  F_CPU=  8 MHz 10485, d.h. 10,485 s. */
		/* max. bei  F_CPU=  5 MHz 16777, d.h. 16,777 s. */
		/* max. bei  F_CPU=  4 MHz 20971, d.h. 20,971 s. */
		/* max. bei  F_CPU= 2,5 MHz 33554, d.h. 33,554 s. */
		/* max. bei  F_CPU=  2 MHz 41943, d.h. 41,943 s. */
		/* max. bei  F_CPU= 1,25 MHz 67108, d.h. 1 min. 7,108 s. */
		/* max. bei  F_CPU=  1 MHz 83886, d.h. 1 min. 23,886 s. */
			/* Beispiel: 	asm volatile("nop");
							meine_verzoegerung_delms(1000);
							asm volatile("nop"); // 1,0 s */

#define meine_verzoegerung_delmks(x) (meine_verzoegerung_delayloop16(((x*(F_CPU/100UL))/40000UL)-1))  
/* x, us, fuer delayloop16 */
		/* max. bei  F_CPU= 20 MHz 13107, d.h. 13,107 ms */
		/* max. bei  F_CPU= 16 MHz 16383, d.h. 16,383 ms */
		/* max. bei  F_CPU= 10 MHz 26214, d.h. 26,2144 ms */
		/* max. bei  F_CPU=  8 MHz 32767, d.h. 32,767 ms */
		/* max. bei  F_CPU=  5 MHz 52428, d.h. 52,428 ms */
		/* max. bei  F_CPU=  4 MHz 65535, d.h. 65,535 ms */
		/* max. bei  F_CPU= 2,5 MHz 104856, d.h. 104,856 ms */
		/* max. bei  F_CPU=  2 MHz 131070, d.h. 131,07 ms */
		/* max. bei  F_CPU= 1,25 MHz 209712, d.h. 209,712 ms */
		/* max. bei  F_CPU= 1 MHz 262140, d.h. 262,140 ms */
			/* Beispiel: 	asm volatile("nop");
							meine_verzoegerung_delmks(16000);
							asm volatile("nop"); // 16 ms */

#define meine_verzoegerung_delcycle(x) (meine_verzoegerung_delayloop8(((x*(F_CPU/3UL))/10000000UL)-1)) 
/* x, 10*us, fuer delayloop8 */
		/* max.384, d.h.  bei  F_CPU= 20 MHz 38,4 us */
		/* max.480, d.h.  bei  F_CPU= 16 MHz 48,0 us */
		/* max.768, d.h.  bei  F_CPU= 10 MHz 76,8 us */
		/* max.960, d.h.  bei  F_CPU=  8 MHz 96,0 us */
		/* max.1536, d.h.  bei  F_CPU=  5 MHz 153,6 us */
		/* max.1920, d.h.  bei  F_CPU=  4 MHz 192,0 us */
		/* max.3072, d.h.  bei  F_CPU= 2,5 MHz 307,2 us */
		/* max.3840, d.h.  bei  F_CPU=  2 MHz 384,0 us */
		/* max.6144, d.h.  bei  F_CPU= 1,25 MHz 614,4 us */
		/* max.7680, d.h.  bei  F_CPU=  1 MHz 768,0 us */
			/* Beispiel: 	asm volatile("nop");
							meine_verzoegerung_delcycle(400);
							asm volatile("nop"); // 40 us */

/* Prototype Inline-Funktionen */

static inline void __attribute__((always_inline))
meine_verzoegerung_delayloop8 (unsigned char count); 
/* Die Schleife dauert  3 * count + 3  Ticks */
/* bei F_CPU 10 MHz max 76,8 mcs */

static inline void __attribute__((always_inline))
meine_verzoegerung_delayloop16 (unsigned int count);
/* Die Schleife dauert  4 * count + 4  Ticks */
/* bei F_CPU 10 MHz max 26,2144 ms */

static inline void __attribute__((always_inline))
meine_verzoegerung_delayloop24 (unsigned long count);
/* Die Schleife dauert  5 * count + 6  Ticks */
/* bei F_CPU 10 MHz max 8,3886 s */

static inline void __attribute__((always_inline))
meine_verzoegerung_delay_sec(int a); 
/* a = Verzoegerung *10 s, max. 32767 = 54 min.36,7 s */

static inline unsigned char __attribute__((always_inline))
meine_verzoegerung_swap (unsigned char x);

/* Beispiel:
PORTD = meine_verzoegerung_swap(PORTD);
meine_verzoegerung_delayloop8(100);
asm volatile("nop");
meine_verzoegerung_delayloop16(2500);
PORTD++;
meine_verzoegerung_delayloop24(100500);
PORTC++;
meine_verzoegerung_delay_sec(15); delay 1,5 sek
 */

/* Inline-Funktionen */

static inline void __attribute__((always_inline))
meine_verzoegerung_delay_sec(int a){ 
/* a = Verzoegerung *10 s, max. 32767 = 54 min.36,7 s */
	for(int i=a;i>=0;i--){
		meine_verzoegerung_delms(100); // 0,1 s
	}
}

static inline void __attribute__((always_inline))
meine_verzoegerung_delayloop8 (unsigned char count)
{
	/* Die Schleife dauert  3 * count + 3  Ticks */
	/* bei F_CPU 10 MHz max 76,8 mcs */
	asm volatile ("1:"           "\n\t"
		      "subi %A0,1" "\n\t"
		      "brcc 1b"
		      : "+d" (count));
}


static inline void __attribute__((always_inline))
meine_verzoegerung_delayloop16 (unsigned int count)
{
    /* Die Schleife dauert  4 * count + 4  Ticks */
	/* bei F_CPU 10 MHz max 26,2144 ms */

    asm volatile ("1:"           "\n\t"
                  "sbiw %0,1"    "\n\t"
                  "brcc 1b"
                  : "+w" (count));
}

static inline void __attribute__((always_inline))
meine_verzoegerung_delayloop24 (unsigned long count)
{
   /* Die Schleife dauert  5 * count + 6  Ticks */
	/* bei F_CPU 10 MHz max 8,3886 s */
	asm volatile ("1:"	   "\n\t"
		      "subi %A0,1" "\n\t"
		      "sbci %B0,0" "\n\t"
		      "sbci %C0,0" "\n\t"
		      "brcc 1b"
		      : "+d" (count));
}

static inline unsigned char __attribute__((always_inline))
meine_verzoegerung_swap (unsigned char x)
{
    asm ("swap %0" : "+r" (x));
    return x;
}


#endif
